#include <windows.h>
#include <stdio.h>
#include <process.h>

unsigned _stdcall ChildThreadProcedure( LPVOID lpSemaphore) {
    // covert parameter to semaphore handle...
    HANDLE hSemaphore = (HANDLE) lpSemaphore;

    // save this thread's id...
    DWORD dwThreadId = GetCurrentThreadId();

    // repeat three times...
    for (int n =0; n < 3; n++) {
        // wait to acquire the semaphore...
        DWORD dwResult = WaitForSingleObject( hSemaphore, INFINITE);

        if (dwResult == WAIT_OBJECT_0) {
            printf("Thread 0x%08x - acquired the semaphore.\n", dwThreadId);
            // hold the semaphore for one-half second and sleep...
            Sleep(500);

            printf("Thread 0x%08x - releasing the semaphore.\n", dwThreadId);
            LONG lPreviousCount;
            ReleaseSemaphore( hSemaphore, 1, &lPreviousCount);
            
            // wait for one-half second before trying again...
            Sleep(500);
        }
        else {
            printf("Thread 0x%08x - error calling WaitForSingleObject().\n", dwThreadId);
            return 0xFFFFFFFF;
        }
    }

    return 0;
}

int main(void) {
    HANDLE hChildThread[3];
    HANDLE hSemaphore;

    // create the semaphore...
    hSemaphore = CreateSemaphore( NULL, 2, 2, NULL);
    if (hSemaphore == NULL) {
        printf("Primary thread - error calling CreateSemaphore().\n");
        exit(0xFFFFFFFF);
    }

    // create the child threads...
    unsigned uUnusedThreadId;
    hChildThread[0] = (HANDLE)_beginthreadex( NULL, 0, ChildThreadProcedure, (void*) hSemaphore, 0, &uUnusedThreadId);
    hChildThread[1] = (HANDLE)_beginthreadex( NULL, 0, ChildThreadProcedure, (void*) hSemaphore, 0, &uUnusedThreadId);
    hChildThread[2] = (HANDLE)_beginthreadex( NULL, 0, ChildThreadProcedure, (void*) hSemaphore, 0, &uUnusedThreadId);
    if (!hChildThread[0] || !hChildThread[1] || !hChildThread[2]) {
        printf("Primary thread - error creating child threads.\n");
        exit(0xFFFFFFFF);
    }

    // wait until the child threads have exited...
    DWORD dwResult = WaitForMultipleObjects( 3, hChildThread, TRUE, INFINITE);
    if (dwResult != WAIT_OBJECT_0) {
        printf("Primary thread - error calling WaitForMultipleObjects().\n");
        exit(0xFFFFFFFF);
    }

    // clean up all handles...
    CloseHandle(hSemaphore);
    CloseHandle(hChildThread[0]);
    CloseHandle(hChildThread[1]);
    CloseHandle(hChildThread[2]);

    return 0;
}

